#![forbid(unsafe_code)]

use std::{fs::create_dir_all, net::SocketAddr, process::Stdio};

use web::state::{SharedState, TemplateState};

mod web;

fn check() {
    let res = std::process::Command::new("python")
        .arg("--version")
        .stdout(Stdio::null())
        .stderr(Stdio::null())
        .spawn();
    if res.is_err() {
        panic!("Python is not installed. Make sure that `python` executable exists in $PATH and is Python 3");
    }
    
    if create_dir_all("/tmp/sandy-tmp").is_err() {
        panic!("Cannot create temporary directory on /tmp/sandy-tmp");
    }
}

#[tokio::main]
async fn main() {
    check();
    
    let state = SharedState {
        template: TemplateState {}
    };
    let routes = web::routes(state);

    println!("Running on 0.0.0.0:80");

    warp::serve(routes).run("0.0.0.0:80".parse::<SocketAddr>().unwrap()).await;
}
