use std::{fs::{self, remove_file}, io::Read, process::{Command, Stdio}};

use rand::{distributions::Alphanumeric, Rng};
use serde::{Serialize, Deserialize};

use warp::{reject::Rejection, reply::{json, with_status, Reply}, Filter, http::StatusCode};

use crate::SharedState;

#[derive(Serialize, Deserialize)]
struct ExecutorData {
    code: String,
    lang: String
}

async fn executor(_state: SharedState, data: ExecutorData) -> Result<Box<dyn Reply>, Rejection> {
    
    if data.lang != "python" {
        return Ok(
            Box::new(
                with_status(
                    json(&"only python supported".to_string()),
                    StatusCode::BAD_REQUEST
                )
            )
        )
    }

    let name = rand::thread_rng()
        .sample_iter(&Alphanumeric)
        .take(64)
        .map(char::from)
        .collect::<String>();
    let path = format!("/tmp/sandy-tmp/{name}");
    
    fs::write(path.clone(), &data.code).unwrap();
    let mut out = Command::new("python")
        .arg(path.clone())
        .stdout(Stdio::piped())
        .spawn()
        .unwrap();
    
    let exit_status = out.wait().unwrap().code().unwrap();

    let mut buf = vec![];
    out.stdout.unwrap().read_to_end(&mut buf).unwrap();

    let mut stdout = String::from_utf8(buf).unwrap();
    stdout += "\n---\n";
    if exit_status != 0 {
        stdout += format!("<span style='color:red;font-weight:bolder'>Command exited with code {}</span>", exit_status).as_str();
    } else {
        stdout += "Command exited with code 0";
    }

    let stdout = stdout.replace('\n', "<br>");
    
    remove_file(path).unwrap();

    Ok(Box::new(warp::reply::json(&stdout)))
}

fn executor_f(state: SharedState) -> impl Filter<Extract = impl Reply, Error = Rejection> + Clone {
    warp::post()
        .map(move || state.clone())
        .and(warp::body::json())
        .and_then(executor)
}

pub fn routes(state: SharedState) -> impl Filter<Extract = impl Reply, Error = Rejection> + Clone {
    executor_f(state)
}